/* -*- mode: C++; tab-width: 8; c-basic-offset: 8; c-file-offsets: ((substatement-open . 0)) -*- */
/**
 * @file	Document.hpp
 * @author	Ingo Luetkebohle (iluetkeb@TechFak.Uni-Bielefeld.de)
 * @date	started at 2004/06/18
 * @version	$Id: Document.hpp 4251 2015-01-14 21:35:25Z rhaschke $
 *
 * @brief       XMLTIO document
 *
 */

#ifndef __XMLTIO_DOCUMENT_HPP
#define __XMLTIO_DOCUMENT_HPP

#include "ID.hpp"
#include "xmltio_config.hpp"

#include <boost/config/requires_threads.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/weak_ptr.hpp>
#include <iostream>

namespace XERCES_CPP_NAMESPACE {
class DOMDocument;
}

namespace xmltio {
class Location;
class XercesDocumentBody;

typedef boost::weak_ptr<XercesDocumentBody> DocumentRef;

class XMLTIO_API TIODocument : public boost::shared_ptr<XercesDocumentBody>
{
public:
	TIODocument() {};
	explicit TIODocument(const std::string& xml);
	explicit TIODocument(const Location& l);
	explicit TIODocument(XercesDocumentBody* doc);
	/// create from document and take ownership!
	explicit TIODocument(XERCES_CPP_NAMESPACE::DOMDocument*);

	Location import(const Location&);
	Location getRootLocation();
	const Location getRootLocation() const;
	/** return the id of the document,
	 *  which is defined as the dbxml:id attribute
	 *   of the root node.
	 *
	 * Note: at the moment, this ID will only be
	 * present after the document has been inserted
	 * into the memory.
	 * */
	ID getID() const;

	// factory methods
	Location createTextLocation(const std::string& xml);
	Location createElement(const std::string& uri,
	                       const std::string& prefix,
	                       const std::string& qname);
	Location createAttribute(const std::string& uri,
	                         const std::string& prefix,
	                         const std::string& qname);

	operator bool() const { return get() != NULL; }

	/** \brief clone the document
	    creates a clone of this TIODocument.
	    @return new cloned TIODocument
	*/
	TIODocument clone() const;
};
std::ostream& operator<<(std::ostream&, const TIODocument&);

// compatibility method
TIODocument parse(const std::string& xml);

// compatibility name
typedef TIODocument DocumentPtr;

} // namespace xmltio

#endif // __XMLTIO_DOCUMENT_HPP
