/* -*- mode: C++; tab-width: 8; c-basic-offset: 8; c-file-offsets: ((substatement-open . 0)) -*- */
/**
 * @file	XercesDocumentBody.hpp
 * @author	Ingo Luetkebohle (iluetkeb@TechFak.Uni-Bielefeld.de)
 * @date	started at 2004/06/18
 * @version	$Id: XercesDocumentBody.hpp 4251 2015-01-14 21:35:25Z rhaschke $
 *
 * @brief       XMLTIO document body based on Xerces-C
 *
 */

#ifndef __XMLTIO_XERCES_DOCUMENT_BODY_HPP
#define __XMLTIO_XERCES_DOCUMENT_BODY_HPP

#include <iosfwd>

#include "Namespaces.hpp"
#include "xmltio_config.hpp"

namespace xmltio {
class TIODocument;

class XMLTIO_API XercesDocumentBody {
private:
	XERCES_CPP_NAMESPACE::DOMDocument* doc_;
	Namespaces nsps_;

public:
	XercesDocumentBody(const std::string& xml);
	XercesDocumentBody(const XercesDocumentBody&);
	XercesDocumentBody(XERCES_CPP_NAMESPACE::DOMDocument*);
	XercesDocumentBody(const XercesDocumentBody& b,
	                   XERCES_CPP_NAMESPACE::DOMNode *subpart);
	virtual ~XercesDocumentBody();

	virtual Namespaces& getNamespaces() {
		return nsps_;
	}

	friend class TIODocument;
	friend std::ostream& operator<<(std::ostream&,
	                                const XercesDocumentBody&);
	friend std::string XMLTIO_API dom2string(const TIODocument&, bool);
};
std::ostream& operator<<(std::ostream&, const XercesDocumentBody&);

} // namespace xmltio

#endif // __XMLTIO_XERCES_DOCUMENT_BODY_HPP
