/* -*- mode: C++; tab-width: 8; c-basic-offset: 8; c-file-offsets: ((substatement-open . 0)) -*- */
/**
 *
 * @file	exception.hpp
 * @author	Ingo Luetkebohle <iluetkeb@TechFak.Uni-Bielefeld.de>
 * @date	started at 2004/07/14
 * @version	$Id: exception.hpp 4251 2015-01-14 21:35:25Z rhaschke $
 *
 * @brief       Exceptions for use within the XML Template I/O library.
 *
 **/

#ifndef __XMLTIO_EXCEPTION_HPP
#define __XMLTIO_EXCEPTION_HPP

#include <exception>
#include <string>
#include <xercesc/dom/DOMException.hpp>
#include "xmltio_config.hpp"

namespace xmltio {
class XPath;

class XMLTIO_API XMLTIO_Exception : public std::exception {
private:
	std::string msg_;
public:
	XMLTIO_Exception(const std::string& msg) : msg_(msg) {}
	XMLTIO_Exception(const std::string& msg, const XERCES_CPP_NAMESPACE::DOMException&);
	XMLTIO_Exception(const XERCES_CPP_NAMESPACE::DOMException&);
	virtual ~XMLTIO_Exception() throw() {};
	virtual const char* what() const throw() {
		return msg_.c_str();
	}
};

class XMLTIO_API ParsingException : public XMLTIO_Exception {
public:
	ParsingException(const std::string& msg,
	                 const std::string& content="") :
		XMLTIO_Exception("parse error: " + msg + ", content='" + content + "'") {};
};
class XMLTIO_API NotFoundException : public XMLTIO_Exception {
public:
	NotFoundException(const std::string& msg,
	                  const XPath& xpath);
	NotFoundException(const XERCES_CPP_NAMESPACE::DOMException& ex) :
		XMLTIO_Exception(ex) {}
	NotFoundException(const std::string& msg) :
		XMLTIO_Exception(msg) {};
};
class XMLTIO_API IDNotFoundException : public NotFoundException {
public:
	IDNotFoundException(const std::exception& msg) :
		NotFoundException(msg.what()) {}
};
class XMLTIO_API MalformedXPathException : public XMLTIO_Exception {
public:
	MalformedXPathException(const std::string& msg)
		: XMLTIO_Exception("malformed XPath:" + msg) {
	}
};
class XMLTIO_API InvalidCharacterException : public XMLTIO_Exception {
public:
	InvalidCharacterException(const std::string& msg, const XERCES_CPP_NAMESPACE::DOMException& ex)
		: XMLTIO_Exception(msg, ex) {}
	InvalidCharacterException(const XERCES_CPP_NAMESPACE::DOMException& ex)
		: XMLTIO_Exception(ex) {}

};
class XMLTIO_API UnsupportedOperationException : public XMLTIO_Exception {
public:
	UnsupportedOperationException(const std::string& msg)
		: XMLTIO_Exception("operation not supported: " + msg) {
	}
	UnsupportedOperationException(const XERCES_CPP_NAMESPACE::DOMException& ex)
		: XMLTIO_Exception(ex) {}
};
class XMLTIO_API ConversionFailedException : public UnsupportedOperationException {
public:
	ConversionFailedException(const std::string& msg)
		: UnsupportedOperationException("conversion failed: " + msg) {};
};
}

#endif //  __XMLTIO_LOCATION_HPP
