/* -*- mode: C++; tab-width: 8; c-basic-offset: 8; c-file-offsets: ((substatement-open . 0)) -*- */

#ifndef __MEMORY_LOG_HPP
#define __MEMORY_LOG_HPP

#ifdef HAVE_LIBLOG4CXX

#undef PACKAGE
#undef VERSION
#include <log4cxx/logger.h>
#include <sstream>

#define GETLOGGER(LOGGER_NAME) namespace {                              \
                               log4cxx::LoggerPtr catlog = log4cxx::Logger::getLogger(LOGGER_NAME);	\
	}

#define DEBUG(DEBUG_LOG_MSG) {                                          \
		if(catlog->isDebugEnabled()) { \
			::std::ostringstream oss; \
			oss << DEBUG_LOG_MSG; \
			catlog->forcedLog(::log4cxx::Level::getDebug(), oss.str()); \
		}}

#define WARN(DEBUG_LOG_MSG) {                                           \
		if(catlog->isWarnEnabled()) { \
			::std::ostringstream oss; \
			oss << DEBUG_LOG_MSG; \
			catlog->forcedLog(::log4cxx::Level::getWarn(), oss.str()); \
		}}

#define INFO(DEBUG_LOG_MSG) {                                           \
		if(catlog->isInfoEnabled()) { \
			::std::ostringstream oss; \
			oss << DEBUG_LOG_MSG; \
			catlog->forcedLog(::log4cxx::Level::getInfo(), oss.str()); \
		}}

#define ERROR(DEBUG_LOG_MSG) {                                          \
		if(catlog->isErrorEnabled()) { \
			::std::ostringstream oss; \
			oss << DEBUG_LOG_MSG; \
			catlog->forcedLog(::log4cxx::Level::getError(), oss.str()); \
		}}

#define FATAL(DEBUG_LOG_MSG) {                                          \
		if(catlog->isFatalEnabled()) { \
			::std::ostringstream oss; \
			oss << DEBUG_LOG_MSG; \
			catlog->forcedLog(::log4cxx::Level::getFatal(), oss.str()); \
		}}

#else

// log4cxx is not available, disable logging macros

#define GETLOGGER(LOGGER_NAME)
#define DEBUG(MSG)
#define INFO(MSG)
#define WARN(MSG)
#define ERROR(MSG)
#define FATAL(MSG)

#endif // HAVE_LOG4CXX

#endif
