/* -*- mode: C++; tab-width: 8; c-basic-offset: 8; c-file-offsets: ((substatement-open . 0)) -*- */

#include "str.hpp"
#include "exception.hpp"
#include "log.hpp"
#include "xinit.hpp"
#include "Document.hpp"
// ouch, this should've stayed private
#include "XercesDocumentBody.hpp"

#include <iostream>
#include <memory>

#include <xercesc/util/XercesVersion.hpp>
#if XERCES_VERSION_MAJOR < 3
#include <xercesc/dom/DOMWriter.hpp>
#else
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOMLSSerializer.hpp>
#include <xercesc/dom/DOMLSOutput.hpp>
#endif

#include <xercesc/dom/DOMImplementationRegistry.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/framework/MemBufFormatTarget.hpp>
#include <xercesc/dom/DOMException.hpp>
#include <xercesc/dom/DOMDocument.hpp>

using namespace std;
using namespace XERCES_CPP_NAMESPACE;

GETLOGGER("xmltio.str");

namespace {
ostream& operator<<(ostream& out, const DOMException& ex)
{
	out << "DOMException[code=" << ex.code
	    << ", msg=" << ex.msg
	    << ']';
	return out;
}
}

namespace xmltio {
static const X PPRINT_FEATURE ("format-pretty-print");

X::X(const std::string& s) {
	getXQ();
	xmlch_ = XMLString::transcode(s.c_str());
}
X::~X() {
	XMLString::release(&xmlch_);
}
NX::NX(const XMLCh* s) {
	getXQ();
	ch_ = XMLString::transcode(s);
}
NX::~NX() {
	XMLString::release(&ch_);
}

string dom2string(const DOMNode* n)
{
	return dom2string(n, false);
}

string dom2string(const DOMNode* n, bool pprint)
{
	static DOMImplementation *impl = 0;
	if (!impl) impl = DOMImplementationRegistry::getDOMImplementation(X("LS"));

	if(!n) {
		throw XMLTIO_Exception("node empty");
	}

	try {
#if XERCES_VERSION_MAJOR < 3
		auto_ptr<DOMWriter> writer(impl->createDOMWriter());
		writer->setEncoding(X("UTF-8"));
		if(pprint && writer->canSetFeature(PPRINT_FEATURE, true)) {
			writer->setFeature (PPRINT_FEATURE, true);
#else
			auto_ptr<DOMLSSerializer> writer (((DOMImplementationLS*)impl)->createLSSerializer());
			if(pprint && writer->getDomConfig()->canSetParameter(PPRINT_FEATURE, true)) {
				writer->getDomConfig()->setParameter (PPRINT_FEATURE, true);
#endif
			}

			MemBufFormatTarget buf;
#if XERCES_VERSION_MAJOR < 3
			writer->writeNode(&buf, *n);
#else
			auto_ptr<DOMLSOutput> output (((DOMImplementationLS*)impl)->createLSOutput());
			output->setEncoding(X("UTF-8"));
			output->setByteStream(&buf);
			writer->write(n, output.get());
#endif
			if(!buf.getRawBuffer()) {
				DEBUG("dom2string empty");
				throw ParsingException("dom2string result empty", "");
			}
			return std::string((char*)buf.getRawBuffer());
		} catch(const DOMException& ex) {
			DEBUG("dom2string failed: " << ex.msg);
			throw XMLTIO_Exception(ex);
		}
	}
	string dom2string(const TIODocument& n)
	{
		std::ostringstream oss;
		oss << n;
		return oss.str();
	}

	string dom2string(const TIODocument& n, bool pprint)
	{
		return dom2string(n.get()->doc_, pprint);
	}

	TIODocument parse(const string& xml)
	{
		return TIODocument(xml);
	}

}
