/* -*- mode: C++; tab-width: 8; c-basic-offset: 8; c-file-offsets: ((substatement-open . 0)) -*- */

#include "XPath.hpp"
#include "Location.hpp"
#include <time.h>
#include <iostream>

using namespace xmltio;
using namespace std;

template<typename F> void bench(const char* tag, int times, F fun)
{
	clock_t start = clock();
	for(int i = 0; i < times ; ++i) {
		fun();
	};
	clock_t end = clock();

	cout << tag << " - elapsed time: "
	     << ((float)(end - start))/CLOCKS_PER_SEC << endl;
};

namespace {
const string foo = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><foo bar=\"none\"/>";
Location lroot(foo, "/");
Location lfoo(foo, "/foo");
XPath afoo("/foo");
XPath rfoo("foo");
XPath bar("@bar");

void simple_absolute() { afoo.evaluate(lroot); }
void simple_relative() { rfoo.evaluate(lroot); }
void simple_attribute_direct() { Location bar(lfoo["bar"]); }
void simple_attribute_xpath() { lfoo[bar]; }
}

int main(int argc, char* argv[])
{
	const int times = 5000;

	bench("simple, absolute", times, simple_absolute);
	bench("simple, relative", times, simple_relative);
	bench("attribute, direct", times, simple_attribute_direct);
	bench("attribute, xpath", times, simple_attribute_xpath);

	return 0;
}
